c     Plasma2d                              ---------------------------|
      SUBROUTINE plasma2d(r,z,psi,Br,Bz,RBphi,omc,Tee,pe2,pi2,aio,ael)
#include "Implic.inc"
c     
c     In which input cylindrical coords r,z (in m)
c     yield outputs psi, the poloidal flux (in webers per radian),
c     Br,Bz (in T) and RBphi (in T-m), and omc in GHz frequency,
c     Te (in keV),
c     \omega_{pe}^2 = pe2 in GHz^2,
c     \omega_{pi}^2 = pi2 in GHz^2,
c     \alpha_{ion}  = 3 \omega_{pi}^2/\omega^2 v_{Ti}^2/\omega^2
c     = aio (in inverse GHz^4)
c     \alpha_{elect}= 3/4 \omega_{pe}^2/\omega_{ce}^2
c     v_{Te}^2/\omega^2
c     = ael (in inverse GHz^4) [must divide by f_{ce}^4]
c     
c     
c     Fields are related to the flux by
c     Br = -(1/R) d psi/d Z
c     Bz =  (1/R) d psi/d R
c     
c     A 'Bean' Equilibrium of TSC is used; up-down symmetric.
c     The grid data file goes from the midplane up.
c     
c     Ignat       TSC     TSC
c     z-numbers   symm    full
c     
c     .   .   .   .   .   .   .   .   . NumZ      nzp     nzp
c     .   .   .   .   .   .   .   .   .           nz      nz
c     .   .   .   .   .   .   .   .   .           nz-1    nz-1
c     .   .   .   .   .   .   .   .   .
c     .   .   .   .   .   .   .   .   .           3
c     mid plane _   _   _   _   _   _   _   _ NumZh     2     (nz+1)/2+1
c     .   .   .   .   .   .   .   .   .           3
c     .   .   .   .   .   .   .   .   .
c     .   .   .   .   .   .   .   .   . 3                 4
c     .   .   .   .   .   .   .   .   . 2         nz      3
c     .   .   .   .   .   .   .   .   . 1         nzp     2
c     
c     1   2   3   4                 NumR      Ignat numbering
c     2   3   4                  nx  nxp      TSC   numbering
c     
c     
c     Flux array numbering: xsv
c psimin                          psilim
c    |                              |
c     .   .   .   .   .   .   .   .   .
c     1   2   3   4            NpsiM1 NpsiJ   Ignat numbering
c     2   3   4   5            npsit  npsip   TSC   numbering 
c     
Comment: From 000431 jardin steve@nersc (ppl) on 06/24/91 at 19:11:01  
c	David:
c       TSC writes out for you the arrays
c       XSV(l),l=2,npsip       poloidal flux
c       anecc(l),l=2,npsip     electron density
c       tekev(l),l=2,npsip     electron temperature
cetc
c       These are all defined at the same locations.  Location 2
c       is near the center (but not exactly at the center) and
c       location npsip is just outside the plasma boundary. 
c
c       TSC wants to read in the arrays
c         
c       POWTSC(l),l=2,npsit
c       CURRTSC(l),l=2,npsit
c  
c       so note that these quantities are defined at the same locations
c       as the ones written out, but that there is one less location read
c       than what is written.  This is because we dont read back in the
c       value of power and current density at location npsip just outside
c       the plasma.
c 
c       note that      npsip = npsit+1
c                      npsim = npsit-1
c
c       depending on how you read your arrays in, you might store
c       and write out your stuff from   1,npsim   instead of 2,npsit.
c       If so , just keep in mind that your quantities are shifted
c       in the index 1 from TSC. 
c                                             Steve (25jun91)
c
c     Identities:   NumZ = nz(full)
c     NumZh= nz(symm)
c     NumR = nx
c     NpsiJ = npsit = npsip-1
c     NumZh=(NumZ+1)/2
c     NumZ = 2*NumZh-1
c     Description of TSC variables:
c     (note--everything in MKS if not specified)
c     
c     achrg:  charge of eqch ion species (hydrogen is 1)
c     amass:  mass of each ion species in grams (H mass is 1.6e-24 GRAM)
c     anecc:  electron density in particles/cc      --> eNeVec
c     anicc:       ion density in particles/cc      --> eNiVec
c     apl:       plasma current in amperes
c     bgzero:    vacuum field strength at rgzero
c     bgzero * rgzero == gzero in other routines
c     Btesl in common /RayWrk/
c     dVlVec: volume of a \psi-surface as passed from TSC: vptemp*(d\psi)
c     iVlVec: integral of volume from center to halfway to next indexed surface
c     gary:   toroidal field function R*Bt (MKS) -- G of psi
c     gpary:  derivative of gary wrt xsv --         G-prime of psi
c     iplim:  limiter switch   pos-plasma rests on limiter   neg-diverted
c     isym:   symmetry option,  0-no symmetry    1- up/down symmetry
c     kcycle: TSC time cycle number (starts at 0)
c     equhd:   80 character description of run from TSC title card
c     npsit:
c     npsitm: number of flux surfaces for surface averaged quantities
c     (note npsitm = npsit-1)
c     |guard point                            |guard point
c     .   .   .   .   .   .   .   .   .   .   .
c     1   2   3         nx-3    nx-1  nx nxp  nxp+1
c     ^                               ^
c     2                              npsit
c     read                             read
c     
c     nspc:   total number of ion species
c     nx:     number of cartesian mesh points in x direction (for PsiGrd)
c     nz:     number of cartesian mesh points in z direction (for PsiGrd)
c     (called nxpl and nzpl in TSC write statements)
c     PARAMETERS:
c     PWORDS:  P       R   number of comment words read in
c     PIMP:     A     E    number of impurities  == nspc - 1
c     PPSI       R   T     number of psi values for fns of psi
c     PNX:        A E      number of x points in PsiGrd grid
c     PNZ:         M       number of z points in PsiGrd grid
c     
c     pary:   pressure (MKS units) --               p of psi
c     ppary:  derivative of pressure wrt xsv --     p-prime of psi
c     psep,xsep,zsep:  flux value,x and z coordinates of separatrix (2)
c     PsiGrd:       poloidal flux per radian
c     psimin:\  poloidal flux per radian at magnetic axis and P/V boundary,
c     psilim:/  defined in common /RayWrk/
c     rgzero: nominal major radius of machine (= xplas)
c     Rmaj in common /RayWrk/
c     rho:    sqrt of the normalized *toroidal* flux
c     tekev:  electron temp in kev                  --> TeeVec
c     tikev:  ion temp in kev
c     times:  TSC problem time in sec
c     vptemp: derivative of volume wrt xsv
c     vptemp: note, this is centered derivative as of Sep93!!
c     vptemp: vol of shell = vptemp(i) * (xsv(i+1) - xsv(i-1))/2.
c     vptemp: name chosen in TSC to distinguish it as a derivative wrt poloidal
c             flux.  TSC mostly works on toroidal flux.
c     voltlp: toroidal loop voltage in volts
c     xary(2):leftmost boundary of cartesian mesh
c     xary(nxp): rightmost boundary of cartesian mesh
c     xmag,zmag:      (x,z) coordinates of magnetic axis
c     xsep:    see psep
c     xsv:     poloidal flux per radian
c     zary(2):   bottom boundary of cartesian mesh
c     zary(nzp):    top boundary of cartesian mesh
c     zsep:    see psep
c     
c     PsiVec is a vector containing the psi values where quantities
c     are given for pressure and diamagnetism
c     ___Vec is explained in the following:
c     RBpVec is a vector containing R*Bphi (phi=toroidal): m-T
c     pe2Vec is a vector containing omega_{pe}^2 in GHz^2 frequency
c     pi2Vec is a vector containing omega_{pi}^2 in GHz^2 frequency
c     TeeVec is a vector of electron temperature in keV
c     AioVec is a vector of thermal correction from ions in GHz^4 frequency
c     
#include "MKSetc.inc"
#include "PIetc.inc"
#include "params.inc"
#include "RayWrk.inc"
#include "ProfBody.inc"
#include "TSCgrap.inc"
#include "plcmx.inc"
      INTEGER RETURN_EVERYTHING, FORCE_RECALC, NO_FORCE, OUT_OF_RANGE
      PARAMETER(RETURN_EVERYTHING = 0, FORCE_RECALC = 1, NO_FORCE = 0,
     ^     OUT_OF_RANGE = -1)
      INTEGER iread, isave, jsave
      DATA isave, jsave
     ^     / OUT_OF_RANGE, OUT_OF_RANGE /
c     initalize out of range
      REAL    psderiv(0:2, 0:2), psimat(0:3, 0:3)
      COMMON / plxxr / psderiv, psimat
c     COMMON / plxxi / isave, jsave ! commented out june17 92 ibm barfed
c     enforce static allocation
      REAL
     ^     HUGE
      REAL
     ^     r,z, psi, Br, Bz, RBphi, omc, Tee, pe2, pi2, aio, ael
      REAL
     ^     Rold,   Zold, psiold,  Brold,  Bzold, omcold
c     DATA  achrg(1),   achrg(2),   achrg(3)        ! These presented here
c    ^     / 1.,         8.,         6./            ! for information purposes.
c           [1] H      [16] O      [12] C           ! Data not actually used
c     DATA  amass(1),   amass(2),   amass(3)        ! in the program.
c    ^     /1.6726e-24,  26.76e-24,  20.07e-24/     ! 
      
      DATA        HUGE  ,  iread /
     ^     CONST(1.0,+30),    0 /
      REAL       ZERO
      DATA       ZERO   /
     &     CONST(0.0,0) /
      
c     
c     ***************************************************************
      if (iread .eq. 1 ) go to 100
      iread = 1
      Rold  = ZERO
      Zold  = ZERO
      Psiold = ZERO
      Brold  = ZERO
      Bzold  = ZERO
      omcold = ZERO
c
c
      
 100  Continue
      If ( r .eq. Rold .and. z .eq. Zold ) then
         Psi    = psiold
         Br     = Brold
         Bz     = Bzold
         omc = omcold
      else 
         Rold = r
         Zold = z
         call grapLSC(RETURN_EVERYTHING, r, z, psi, psderiv, NO_FORCE,
     ^        rmin, dr, zmin, dz, isave, jsave, nx, NumZ, PNX, PsiGrd,
     ^        psimat)
         Br = - psderiv(0, 1) / r
         Bz = psderiv(1, 0) / r
      endif
      call plasma1d (psi, psiold, RBphi,Tee,pe2,pi2,aio,ael)
      omc = OmcFac * SQRT( Br**2 + Bz**2 + (RBphi/r)**2 )
      Ael = Ael / omc**4
      Te = Tee
c
      Brold  = Br
      Bzold  = Bz
      omcold = omc
      return
      END
c     
c     
c     plasma2d ends                         ---------------------------|
      
c     
      SUBROUTINE plasma1d (psi, psiold, RBphi,Tee,pe2,pi2,aio,ael)
#include "Implic.inc"
#include "params.inc"
#include "ProfBody.inc"
#include "TSCgrap.inc"
      REAL    psderiv(0:2, 0:2), psimat(0:3, 0:3)
      COMMON / plxxr / psderiv, psimat
      INTEGER jRBphi, jpe2, jTee, jpi2, jAio, jAel, jVpr,
     ^     RECALC, first_call
      PARAMETER(RECALC = -1)
      DATA jRBphi, jpe2, jTee, jpi2, jAio, jAel, jVpr  /
     ^     RECALC, RECALC, RECALC, 
     ^     RECALC, RECALC, RECALC, RECALC /
      DATA first_call / TRUE /
      REAL
     ^     RBphipr, pe2pr, Teepr, pi2pr, Aiopr, Aelpr, 
     ^ psl,psi, psiold, RBphi, Tee, pe2, pi2, aio, ael,
     ^     RBphio, pe2old, Teeold, pi2old, Aioold, Aelold
      DATA RBphio, pe2old, Teeold, pi2old, Aioold, Aelold /
     ^     0,       0,      0,      0,       0,     0    /
      REAL       ZERO
      DATA       ZERO   /
     &     CONST(0.0,0) /
      if(first_call .eq. TRUE)then
         first_call = FALSE
         Psiold = ZERO
         RBphio = ZERO
         pe2old = ZERO
         Teeold = ZERO
         pi2old = ZERO
         Aioold = ZERO
         Aelold = ZERO
      endif
      
      if(psi .eq. psiold)then
         RBphi  = RBphio
         pe2    = pe2old
         Tee    = Teeold
         pi2    = pi2old
         Aio    = Aioold
         Ael    = Aelold
         return
      endif
      
      call grnu1d(NpsiJ, PsiVec, RBpVec, jRBphi, RBPhiCoefs,
     ^     psi, RBphi, RBphipr)
c     
c     enforce bounds on psi !!!!!!!!!!!!!!
      psi =  max (psiminx, psi)
c     psi =  min (psimaxx, psi)
      psl =  min (psimaxx, psi)
c     stop forcing bounds on psi on the maximum
c     enforce bounds on psi !!!!!!!!!!!!!!
c
      call grnu1d(NpsiJ, PsiVec, pe2Vec, jpe2  , pe2Coefs,
     ^     psl, pe2  , pe2pr)
      call grnu1d(NpsiJ, PsiVec, TeeVec, jTee  , TeeCoefs,
     ^     psl, Tee  , Teepr)
      call grnu1d(NpsiJ, PsiVec, pi2Vec, jpi2  , pi2Coefs,
     ^     psl, pi2  , pi2pr)
      call grnu1d(NpsiJ, PsiVec, AioVec, jAio  , AioCoefs,
     ^     psl, Aio  , Aiopr)
      call grnu1d(NpsiJ, PsiVec, AelVec, jAel  , AelCoefs,
     ^     psl, Ael  , Aelpr)

      Psiold = Psi
      RBphio = RBphi
      pe2old = pe2
      Teeold = Tee
      pi2old = pi2
      Aioold = Aio
      Aelold = Ael
      return
      END
c
c     plsgrd.f(or)  begins                  ---------------------------|
c                                                                      |
c                                                                      |

      SUBROUTINE plsgrd
c     set up grids PsiGrd, xary, zary, xsv
c     isym: symmetry option,  0-no symmetry    1- up/down symmetry
#include "Implic.inc"
#include "params.inc"
#include "MKSetc.inc"
#include "ProfBody.inc"
#include "RayWrk.inc"
      INTEGER i, j
#include "TSCgrap.inc"
#include "plcmx.inc"
      REAL
     ^     TwoH, TwoK

      if (isym .eq. 1) then
            do  10 j=1,NumZh-1
                do  10 i=1,NumR
                   PsiGrd(i, j) = PsiGrd(i, NumZ-(j-1))
 10         continue
      endif
c
 
 
20    TwoH = (xary(NumR)-xary(1))/AREAL(NumR-1)

      TwoK = (zary(NumZ)-zary(1))/AREAL(NumZ-1)
      dr = TwoH
      dz = Twok

 
      Do 25 i = 2, NumR
      xary(i)  =  xary(1) + AREAL(i-1) * TwoH
25    continue
      Do 30 j = 2, NumZ
      zary(j)  =  zary(1) + AREAL(j-1) * TwoK
30    continue
 
      return
      END


c                                                                      |
c                                                                      |
c     plsgrd.f(or)  ends                    ---------------------------|

c     RdTSC.f(or)  begins                  ----------------------------|
c                                                                      |
c                                                                      |

      SUBROUTINE rdTSC
c     read equilibrium from TSC generated disk file
#include "Implic.inc"
#include "params.inc"
#include "Doflags.inc"
#include "tscunits.inc"
#include "MKSetc.inc"
#include "PIetc.inc"
#include "ProfBody.inc"
#include "RayWrk.inc"
      INTEGER i, j, n, l, npsiPut
      REAL    voltPut(*)
#include "TSCgrap.inc"
#include "plcmx.inc"
#include "Escan.inc"
      CHARACTER*8  name(10), timhd(2)
      REAL    psderiv(0:2, 0:2), psimat(0:3, 0:3)
      COMMON / plxxr / psderiv, psimat
      REAL   nePeak, TePeak, BoPeak, neEdge
      REAL   neRatio,TeRatio,BoRatio, IpRatio
      REAL   ZeroNe, ZeroTe
      REAL   PsiVal, ssqrt2, TBTemp
      DATA       ZeroNe  ,       ZeroTe   / 
     ^     CONST(1.0,+09), CONST(1.0,-02) /
      REAL       ZERO
      DATA       ZERO   /
     ^     CONST(0.0,0) /
c     pe2Fac pi2Fac convert density in ^14 cm^-3 or ^20 m^-3
c     and frequency in GHz into
c     \omega_{pe}^2 / \omega^2 = pe2Fac * n_e[14]/f^2[GHz]
c     \omega_{pi}^2 / \omega^2 = pi2Fac * n_i[14]/f^2[GHz]
c     pe2Vec is a quantity that must be divided by fghz^2 to get:
c                                                     \omega_{pe}^2 / \omega^2 
c     
c     AelFac AioFac convert density,temperature,mass to thermal terms
c     OmcFac converts total magnetic field to GHz frequency
c     ceiFac convert B , ion density and charge to
c     \omega_{ce} \omega_{ci} /f^2[GHz]
c     TBTemp introduced by TB Terpstra to avoid a divide by zero with
c     the Alpha Open VMS systems in Feb 94

      pe2Fac  =( ECOULB * CLIGHT )**2/ ( PI * ELECMS ) * CONST(1.,+4)
      pe2Fac14=( ECOULB * CLIGHT )**2/ ( PI * ELECMS ) * CONST(1.,-10)
      pi2Fac  = pe2Fac * (ELECMS/PROTMS)               * CONST(1.,-4)
      AioFac  = 3.   * pi2Fac * ECOULB/PROTMS/TWOPI**2 * CONST(1.,-7)
      AelFac  = 0.75 * pe2Fac * ECOULB/ELECMS/TWOPI**2 * CONST(1.,-3)
      OmcFac  =                 ECOULB/ELECMS/TWOPI    * CONST(1.,+3)
      ceiFac  =( ECOULB/TWOPI )**2/ELECMS/PROTMS       * CONST(1.,+2)
c
c
c                                       Make sure we do not hit EOF
      rewind (unit = nTSCwrit)
c
      read(nTSCwrit,2201,end=98,err=99) (equhd(i),i=1,10)
      read(nTSCwrit,2202,end=98,err=99) npsitm,nspc,kcycle,times
c
            write(timhd( 1),'('' time = '')')
            write(timhd( 2),'(f8.4)') times     
            EquTitle = equhd(1)/ /equhd(2)/ /equhd(3)/ /
     ^                 equhd(4)/ /equhd(5)/ /equhd(6)/ /
     ^                 equhd(7)/ /equhd(8)/ /equhd(9)/ /equhd(10)/ /
     ^                 timhd(1)/ /timhd(2)

            NpsiJ    =   npsitm
            NpsiM1  =   npsitm-1
            if (NpsiJ .gt. PPSI)  
     ^                    call LSCstop (' NpsiJ .gt. PPSI on read-in')
            if (nspc .gt. PIMP+1)
     ^                    call LSCstop (' nspc .gt. PIMP+1 on read-in')
            if (iError .gt. 0 ) return
c
      read(nTSCwrit,2203,end=98,err=99)( anecc(l),l=1,NpsiJ)
      read(nTSCwrit,2203,end=98,err=99)( tekev(l),l=1,NpsiJ)
      read(nTSCwrit,2203,end=98,err=99)((anicc(l,n),l=1,NpsiJ),n=1,nspc)
      read(nTSCwrit,2203,end=98,err=99)((tikev(l,n),l=1,NpsiJ),n=1,nspc)
            anecc(NpsiJ)  =  min (anecc(NpsiJ),  ZeroNe)
            tekev(NpsiJ)  =  min (tekev(NpsiJ),  ZeroTe)
            anicc(NpsiJ,1)=  min (anicc(NpsiJ,1),ZeroNe)
            tikev(NpsiJ,1)=  min (tikev(NpsiJ,1),ZeroTe)
      read(nTSCwrit,2203,end=98,err=99) (amass(n),n=1,nspc)
      read(nTSCwrit,2203,end=98,err=99) (achrg(n),n=1,nspc)
c
      read(nTSCwrit,2203,end=98,err=99) (voltlp(l),l=1,NpsiJ)
      read(nTSCwrit,2203,end=98,err=99) (rho(l),l=1,NpsiJ)
      read(nTSCwrit,2203,end=98,err=99) (vptemp(l),l=1,NpsiJ)
      read(nTSCwrit,2203,end=98,err=99) (xsv(l),l=1,NpsiJ)
            psiminx = xsv(1)
            psimaxx = xsv(NpsiJ)
      read(nTSCwrit,2203,end=98,err=99) (pary(l),l=1,NpsiJ)
      read(nTSCwrit,2203,end=98,err=99) (ppary(l),l=1,NpsiJ)
      read(nTSCwrit,2203,end=98,err=99) (gary(l),l=1,NpsiJ)
      read(nTSCwrit,2203,end=98,err=99) (gpary(l),l=1,NpsiJ)
      read(nTSCwrit,2204,end=98,err=99) nx,nz,isym,iplim
              if (nx  .gt. PNX) 
     ^                call LSCstop (' nx .gt. PNX on read-in')
      read(nTSCwrit,2203,end=98,err=99)xary(1),xary(nx),zary(1),zary(nz)
c
            if (isym .eq. 0) then 
c             call LSCwarn (' isym must be 1 for up/dn symmetry')
              if (nz  .gt. PNZ) 
     ^                call LSCstop (' nz .gt. PNZ on read-in')
              NumR    =   nx
              NumZh   =  (nz+1)/2
              NumZ    =   nz
              Rmin    =   xary(1)
              Rmax    =   xary(NumR)
              Zmin    =   zary(1)
              Zmax    =   zary(NumZ)
            else if (isym .eq. 1) then
              NumR    =   nx
              NumZh   =   nz
              NumZ    = 2*nz - 1
              if (NumZ.gt. PNZ) 
     ^                call LSCstop (' NumZ .gt. PNZ on read-in')
              zary(NumZ )=zary(nz)
              zary(NumZh)=zary(1)
              zary(1)    =-zary(NumZ)
              Rmin    =   xary(1)
              Rmax    =   xary(NumR)
              Zmin    =  -zary(NumZ)
              Zmax    =   zary(NumZ)

            else
              call LSCstop (' isym must be 1 for up/dn symmetry, or 0')
              return
            endif
c
      if (iError .gt. 0 ) return
c

c
c
      read(nTSCwrit,2203,end=98,err=99) psimin,psilim,xmag,zmag
      read(nTSCwrit,2203,end=98,err=99) rgzero,bgzero,apl
c
            Rmaj    =  rgzero
            Rmag    =  xmag
            Btesl   =  bgzero
            RBphi0  =  rgzero * bgzero
c
              l = 1
              dVlVec(l) = vptemp(l)* (+0.5*(xsv(l+1) + xsv(l))
     ^        - psimin ) 
              iVlVec(l) = dVlVec(l)
              MidVec(l)= 0.5*(xsv(l+1)+xsv(l))
            do 6 l=2,NpsiJ-1
              dVlVec(l) = vptemp(l)*( xsv(l+1) - xsv(l-1) )*0.5
              iVlVec(l) = iVlVec(l-1) + dVlVec(l)
              MidVec(l)= 0.5*(xsv(l+1)+xsv(l))
 6          continue
              l = NpsiJ
              dVlVec(l) = vptemp(l)* 0.5*(xsv(l) - xsv(l-1))
              iVlVec(l) = iVlVec(l-1) + dVlVec(l)
              MidVec(l)= xsv(l)
            do 7 l=1,NpsiJ
              EdcVec(l) = voltlp(l)/(TWOPI * Rmaj)
 7          continue
            if(iEdc .eq. TRUE)then
               do 77 l = 1, NpsiJ
                  EdcVec(l) = Edcinp
 77            continue
            endif

      read(nTSCwrit,2203,end=98,err=99) psep(1),xsep(1),zsep(1)
      read(nTSCwrit,2203,end=98,err=99) psep(2),xsep(2),zsep(2)
      if( isym .eq. 1) then
      read(nTSCwrit,2203,end=98,err=99) ((PsiGrd(i, j) ,i=1,NumR),
     ^     j=NumZh,NumZ)
      else if (isym .eq. 0) then
      read(nTSCwrit,2203,end=98,err=99) ((PsiGrd(i, j) ,i=1,NumR),
     ^     j=    1,NumZ)
      else
        call LSCstop (' isym must be 1 for up/dn symmetry, or 0')
        return
      endif
      goto 100
c
c     Die gracefully on Read Error Here ............
c                                                  .
 98   call LSCstop(' EOF reached in RdTSC ')
      return
 99   call LSCstop(' read error  in RdTSC ')
      return
c                                                  .
c     Die gracefully on Read Error Here ............
c
 100  continue
c
c
 2201 format(10a8)
 2202 format(3i5,e16.6)
 2203 format(5e16.6)
 2204 format(4i5)
c 
c
c           enmax = anecc(1)
c           Te0   = tekev(1)
c           Ti0   = tikev(1,1)
c
            do 1100 l=1,NpsiJ
            pi2Vec(l) = ZERO
            AioVec(l) = ZERO
            pe2Vec(l) = pe2Fac * (anecc(l)  /CONST(1.,14))
            AelVec(l) = AelFac * (anecc(l)  /CONST(1.,14)) * tekev(l)
c
            do 1100 n=1,nspc
            TBTemp = ((amass(n)/PROTMS)*CONST(1.,24))
            pi2Vec(l) = pi2Vec(l)
     ^              + pi2Fac * (anicc(l,n)/CONST(1.,14)) * achrg(n)**2
     ^              / TBTemp
c    ^              / (amass(n)/PROTMS*1.e24)  ! before Feb94
            AioVec(l) = AioVec(l)
     ^              + AioFac * (anicc(l,n)/CONST(1.,14)) * achrg(n)**2
     ^              / (TBTemp*TBTemp)             * tikev(l,n)
c    ^              / (amass(n)/PROTMS*1.e+24)**2  * tikev(l,n) ! before Feb94
 1100       continue
c
            wcei2   =  ZERO
            do 1105 n=1,nspc
            TBTemp = ((amass(n)/PROTMS)*CONST(1.,24))
            wcei2      = wcei2
     ^                 + ceiFac * anicc(1,n)/anecc(1) * achrg(n)**2
     ^                 / TBTemp 
c    ^                 / (amass(n)/PROTMS*1.e24) ! before Feb94
 1105       continue
            wcei2   = wcei2 * Btesl**2
            capo2   = wcei2 / fghz**2
c
c
c
            pe2min = pe2Vec(NpsiJ - 1)
c 
c                                       Make sure the update is at BOF
c
      rewind (unit = nTSCwrit)
c
c
      call plsgrd
c                                       plsgrd makes symmetry up-down

c     .                                 lcfs: last closed flux surface;
c     .                                 however, it is the 97% flux surface
      PsiVal = PsiLim
     ^  -0.03*(PsiLim-PsiMin)
c
c     .                                 Rmag not Rmaj
c     .                                 The 0.95 stays away avoids confusion
c     .                                 at the exact center where there can be
c     .                                 a tiny dip in psi (numerical noise).
      Rlcfs(1) = Rmag * 0.95
      Zlcfs(1) = zmag
      call PsiSur ( PsiVal , Rlcfs , Zlcfs , ISIZElcfs , Nlcfs )
      Rlcfs(Nlcfs) = Rlcfs(1)
      Zlcfs(Nlcfs) = Zlcfs(1)
          RlcfsMax=Rmaj
          RlcfsMin=Rmaj
          ZlcfsMin=Zlcfs(1)
          ZlcfsMax=Zlcfs(1)
      do 1200 i=1,Nlcfs
          if(Rlcfs(i).gt.RlcfsMax)RlcfsMax=Rlcfs(i)
          if(Rlcfs(i).lt.RlcfsMin)RlcfsMin=Rlcfs(i)
          if(Zlcfs(i).gt.ZlcfsMax)ZlcfsMax=Zlcfs(i)
          if(Zlcfs(i).lt.ZlcfsMin)ZlcfsMin=Zlcfs(i)
 1200  continue

      RlcfsMax = RlcfsMax*1.01
      RlcfsMin = RlcfsMin*0.99
      ZlcfsMax = ZlcfsMax*1.01
      ZlcfsMin = ZlcfsMin*1.01
c     .                                 Keep the lcfs inside the grid!
      RlcfsMax =  min (RlcfsMax, Rmax)
      RlcfsMin =  max (RlcfsMin, Rmin)
      ZlcfsMax =  min (ZlcfsMax, Zmax)
      ZlcfsMin =  max (ZlcfsMin, Zmin)

      return
      ENTRY PutVinLSC(voltPut,npsiPut)
            do 2007 l=1,NpsiJ
              EdcVec(l) = voltPut(l+1)/(TWOPI * Rmaj)
 2007       continue
      return 

      END
c
      
c                                                                      |
c                                                                      |
c     RdTSC.f(or)  ends                     ---------------------------|
c                                                                      |
c                                                                      |
c                                                                      |
c                                                                      |
c     file: grd.F   ends                    ---------------------------|









