c     SBlsc.f(or)  begins                  ----------------------------|
c                                                                      |
c                                                                      |

      PROGRAM SBlsc
c     write equilibrium for stand alone LSC tests, as reqested by 
c     Stefano Bernabei, particularly for the trip to China June 1994

      IMPLICIT NONE
#include "params.inc"
#include "tscunits.inc"
#include "ProfBody.inc"
#include "RayWrk.inc"
      INTEGER i, j, n, l
#include "TSCgrap.inc"
#include "plcmx.inc"
#include "Escan.inc"
      REAL    CLIGHT, ECOULB, ELECMS, PROTMS 
      DATA    CLIGHT, ECOULB, ELECMS, PROTMS /
     ^        2.9979, 1.6022, 9.1095, 1.6726 /
C            10^8 m/s 10^-19c 10^-28g 10^-24g
c
c     We circularize the plasma.  If  for r <= a 
c        J(r) = Jo (1 - (r/a)^2 )^p, that is putting z==(r/a)^2,
c        J(r) = Jo (1 -     z^2 )^p, then
c        B(r) = muo Ip/(2 pi a) (1/(r/a)) [1 - (1 -(r/a)^2 )^(p+1)] , or
c        B(r) = muo Ip/(2 pi a) (1/z)     [1 - (1 -      z )^(p+1)] , and
c      psi(r) = [2 pi] R a muo Ip/(2 pi a) (p+1)/2 z \times
c                                         {1 -
c                          p   z/(2*2)    (1 -
c                         (p-1)z/(3*(3/2))(1 -
c                         (p-2)z/(4*(4/3))(1 - ....)))} 
c      while for r > a
c        J(r) = 0
c        B(r) = B(a) (a/r)
c      psi(r) = psi(a) 
c      psi(r) = psi(a) + [2 pi]  R a muo Ip/(2 pi a) alog (r/a)
c
c     q(r)/q(a) = z / [ 1 - (1-z)^{p+1} ]  
c               = 1 / (p+1) as r --> zero
c     Therefore, p should normally be 2 or more

c
c     SUBROUTINE GetPsiCirc(r,a,Rmaj,BofA,power,BofR,volume,psi,ifail)
c     call       GetPsiCirc(r,a,Rmaj,BofA,power,BofR,volume,psi,ifail)
c
c     Since this code uses *flux per radian*, the [2 pi] is coded as 1.
c
c     achrg:  charge of eqch ion species (hydrogen is 1)
c     amass: m ass of each ion species in grams (H mass is 1.6e-24 GRAM)
c     anecc:  electron density in particles/cc      --> eNeVec
c     anicc:       ion density in particles/cc      --> eNiVec
c     apl:       plasma current in amperes
c     bgzero:    vacuum field strength at rgzero
c     bgzero * rgzero == gzero in other routines
c     Btesl in common /RayWrk/
c     dVlVec: volume of a \psi-surface as passed from TSC: vptemp*(d\psi)
c     iVlVec: integral of volume from center to halfway to next indexed surface
c     gary:   toroidal field function R*Bt (MKS) -- G of psi
c     gpary:  derivative of gary wrt xsv --         G-prime of psi
c     iplim:  limiter switch   pos-plasma rests on limiter   neg-diverted
c     isym:   symmetry option,  0-no symmetry    1- up/down symmetry
c     kcycle: TSC time cycle number (starts at 0)
c     equhd:   80 character description of run from TSC title card
c     npsit:
c     npsitm: number of flux surfaces for surface averaged quantities
c     (note npsitm = npsit-1)
c     |guard point                            |guard point
c     .   .   .   .   .   .   .   .   .   .   .
c     1   2   3         nx-3    nx-1  nx nxp  nxp+1
c     ^                               ^
c     2                              npsit
c     read                             read
c     
c     nspc:   total number of ion species
c     nx:     number of cartesian mesh points in x direction (for PsiGrd)
c     nz:     number of cartesian mesh points in z direction (for PsiGrd)
c     (called nxpl and nzpl in TSC write statements)
c     PARAMETERS:
c     PWORDS:  P       R   number of comment words read in
c     PIMP:     A     E    number of impurities  == nspc - 1
c     PPSI       R   T     number of psi values for fns of psi
c     PNX:        A E      number of x points in PsiGrd grid
c     PNZ:         M       number of z points in PsiGrd grid
c     
c     pary:   pressure (MKS units) --               p of psi
c     ppary:  derivative of pressure wrt xsv --     p-prime of psi
c     psep,xsep,zsep:  flux value,x and z coordinates of separatrix (2)
c     PsiGrd:       poloidal flux per radian
c     psimin:\  poloidal flux per radian at magnetic axis and P/V boundary,
c     psilim:/  defined in common /RayWrk/
c     rgzero: nominal major radius of machine (= xplas)
c     Rmaj in common /RayWrk/
c     rho:    sqrt of the normalized *toroidal* flux
c     tekev:  electron temp in kev                  --> TeeVec
c     tikev:  ion temp in kev
c     times:  TSC problem time in sec
c     vptemp: derivative of volume wrt xsv
c     vptemp: note, this is centered derivative as of Sep93!!
c     vptemp: vol of shell = vptemp(i) * (xsv(i+1) - xsv(i-1))/2.
c     vptemp: name chosen in TSC to distinguish it as a derivative wrt poloidal
c             flux.  TSC mostly works on toroidal flux.
c     voltlp: toroidal loop voltage in volts
c     xary(2):leftmost boundary of cartesian mesh
c     xary(nxp): rightmost boundary of cartesian mesh
c     xmag,zmag:      (x,z) coordinates of magnetic axis
c     xsep:    see psep
c     xsv:     poloidal flux per radian FLUX PER RADIAN!
c     zary(2):   bottom boundary of cartesian mesh
c     zary(nzp):    top boundary of cartesian mesh
c     zsep:    see psep
c     
c     PsiVec is a vector containing the psi values where quantities
c     are given for pressure and diamagnetism
c     ___Vec is explained in the following:
c     RBpVec is a vector containing R*Bphi (phi=toroidal): m-T
c     pe2Vec is a vector containing omega_{pe}^2 in GHz^2 frequency
c     pi2Vec is a vector containing omega_{pi}^2 in GHz^2 frequency
c     TeeVec is a vector of electron temperature in keV
c     AioVec is a vector of thermal correction from ions in GHz^4 frequency

      CHARACTER*8  name(10)
      CHARACTER*4 MACHINE
      REAL
     ^     psderiv(0:2, 0:2), psimat(0:3, 0:3), dpsi
      COMMON / plxxr / psderiv, psimat, dpsi
      REAL   nePeak, TePeak, BoPeak, neEdge
      REAL   neRatio,TeRatio,BoRatio, IpRatio
      REAL   ZeroNe, ZeroTe
      REAL   PsiVal

      INTEGER ifail
      REAL    rpower
      REAL    amin, alim, bofa, bofr, volume, psi, r
      REAL    xdum(PPSI), ydum(PPSI)
      REAL    ane0,ani1,ani2,te0,ti1,ti2,amas1,amas2,chrg1,chrg2
      REAL    expn,expt
      REAL    psiupper, psilower, vupper, vlower, vprime

      DATA   ZeroNe, ZeroTe / 1.0e+09, 1.0e-02 /
c     pe2Fac pi2Fac convert density in ^14 cm^-3 or ^20 m^-3
c     and frequency in GHz into
c     \omega_{pe}^2 / \omega^2 = pe2Fac * n_e[14]/f^2[GHz]
c     \omega_{pi}^2 / \omega^2 = pi2Fac * n_i[14]/f^2[GHz]
c     pe2Vec is a quantity that must be divided by fghz^2 to get:
c                                                     \omega_{pe}^2 / \omega^2 
c     
c     MACHINE = 'HT-7'
      MACHINE = 'TFTR'
c     MACHINE = 'NSTX'
c
c     .............................................................
c
c     CHINESE PARAMETERS:
      if(MACHINE .eq. 'HT-7') then
      open(unit=7,file='HT-7.d',status='unknown')
      write(7,'(''Test Equilibrium for HT-7 for China Trip'')')
      bgzero = 2.5
      rgzero = 1.22
      xmag   = 1.22
      amin   = 0.28
      alim   = 0.33
      apl    = 0.250 * 1.e+6
      nspc   =   1
      ane0   = 2.0e+13
      ani1   = 2.0e+13
      ani2   = 2.0e+11
      te0    = 1.5
      ti1    = 1.5
      ti2    = 1.5
      amas1  = PROTMS*1.0e-24*1.
      amas2  = PROTMS*1.0e-24*2.
      chrg1  = 1.0
      chrg2  = 1.0
      expn   = 1.0
      expt   = 1.0
      rpower     = 2.0
c 
c     NSTX PARAMETERS
c
      else if(MACHINE .eq. 'NSTX')then
      open(unit=7,file='NSTXcirc.d',status='unknown')
      write(7,'(''Test Equilibrium for NSTX Current Drive Studies  '')')
      bgzero = 0.25
      rgzero = 0.80
      xmag   = 0.80
      amin   = 0.64
      alim   = 0.65
      apl    = 0.400 * 1.e+6
      nspc   =   1
      ane0   = 5.0e+13
      ani1   = 5.0e+13
      ani2   = 5.0e+13
      te0    = 1.0
      ti1    = 1.0
      ti2    = 1.0
      amas1  = PROTMS*1.0e-24*2.
      amas2  = PROTMS*1.0e-24*1.
      chrg1  = 1.0
      chrg2  = 1.0
      expn   = 2.0
      expt   = 2.0
      rpower     = 2.0
c
c     TFTR PARAMETERS:
c
      else if(MACHINE .eq. 'TFTR')then
      open(unit=7,file='TFTRcirc.d',status='unknown')
      write(7,'(''Test Equilibrium for TFTR Current Drive Studies  '')')
      bgzero = 4.0
      rgzero = 2.5
      xmag   = 2.5
      amin   = 0.85
      alim   = 0.90
      apl    = 1.000 * 1.e+6
      nspc   =   2
      ane0   = 5.0e+13
      ani1   = 5.0e+13
      ani2   = 5.0e+13
      te0    = 10.
      ti1    = 10.
      ti2    = 10.
      amas1  = PROTMS*1.0e-24*2.
      amas2  = PROTMS*1.0e-24*3.
      chrg1  = 1.0
      chrg2  = 1.0
      expn   = 2.0
      expt   = 2.0
      rpower     = 2.0
      else
         stop ' no machine defined: try NSTX, TFTR, or HY-7'
      endif

      Rmaj   =  rgzero
      Rmag   =  xmag
      Btesl  =  bgzero
      RBphi0 =  rgzero * bgzero
      Rmin   =  Rmaj - alim
      Rmax   =  Rmaj + alim
      Zmax   =  + alim
      Zmin   =  - alim
      psep(1)= 0.00
      xsep(1)= 0.00
      zsep(1)= 0.00
      psep(2)= 0.00
      xsep(2)= 0.00
      zsep(2)= 0.00

            npsitm   = 40
            NpsiJ    =   npsitm            
            NpsiM1   =   npsitm-1
            kcycle   =   100
            times    =   0.0
            nx       =   50
            nz       =   50
            isym     =   0
            iplim    =   1
            if (NpsiJ .gt. PPSI)  stop 'NpsiJ too large'
            if (nspc .gt. PIMP+1) stop 'nspc too large'
            if (nx  .gt. PNX)     stop 'nx too large'
            if (nz  .gt. PNZ)     stop 'nz too large'
            NumR    =   nx
            NumZh   =  (nz+1)/2
            NumZ    =   nz

      do i=1,NumR
         xary(i) = Rmin + float(i-1)*(Rmax-Rmin)/float(NumR-1)
      enddo
      do i=1,NumZ
         zary(i) = Zmin + float(i-1)*(Zmax-Zmin)/float(NumZ-1)
      enddo

      BofA       = 2. * 1.e-7 * apl / amin

      call       GetPsiCirc(amin,amin,Rmaj,BofA,rpower,
     ^                      BofR,volume,psi,ifail)
      psilim = psi*0.99
      psimin = 0.00
      xsv(1) = psi*0.01
      xsv(NpsiJ) = psi
      
      do l=1,NpsiJ
         anecc(l)   = ane0   *( 1. -float(l-1)/float(NpsiJ-1))**expn 
     ^                + ZeroNe
         anicc(l,1) = ani1   *( 1. -float(l-1)/float(NpsiJ-1))**expn
     ^                + ZeroNe/2.
         anicc(l,2) = ani2   *( 1. -float(l-1)/float(NpsiJ-1))**expn
     ^                + ZeroNe/2.
         tekev(l)   = te0    *( 1. -float(l-1)/float(NpsiJ-1))**expt
     ^                + ZeroTe
         tikev(l,1) = ti1    *( 1. -float(l-1)/float(NpsiJ-1))**expt
     ^                + ZeroTe
         tikev(l,2) = ti2    *( 1. -float(l-1)/float(NpsiJ-1))**expt
     ^                + ZeroTe
         xsv(l)     = xsv(1) + 
     ^          (xsv(NpsiJ)-xsv(1))*float(l-1)/float(NpsiJ-1)
         voltlp(l)  = 0.00
         gary(l)    = RBphi0
         gpary(l)   = 0.00
         pary(l)    = 0.00
         ppary(l)   = 0.00
         vptemp(l)  = 0.00
         rho(l)     = 0.00
      enddo
c
         amass(1)   = amas1
         amass(2)   = amas2
         achrg(1)   = chrg1
         achrg(2)   = chrg2
      do i=1,nx
         do j=1,nz
            r = sqrt((xary(i) - Rmaj)**2 + Zary(j)**2)
            call       GetPsiCirc(r,amin,Rmaj,BofA,rpower,
     ^                            BofR,volume,psi,ifail)
            PsiGrd(i,j) = psi
         enddo
      enddo
c
      do i=1,PPSI
         r = amin*0.99*float(i-1)/float(PPSI-1)
         call GetPsiCirc(r,amin,Rmaj,BofA,rpower,
     ^                   BofR,volume,psi,ifail)
         xdum(i) = psi
         ydum(i) = volume
      enddo
c
      do i=2,NpsiJ-1
         psiupper = 0.5*(xsv(i+1) + xsv(i  ))
         psilower = 0.5*(xsv(i  ) + xsv(i-1))
         call linr1d(PPSI, xdum, ydum, psiupper, vupper, vprime )
         call linr1d(PPSI, xdum, ydum, psilower, vlower, vprime )
         vptemp(i) = vupper-vlower
      enddo

         i = 1
         psiupper = 0.5*(xsv(i+1) + xsv(i  ))
         call linr1d(PPSI, xdum, ydum, psiupper, vupper, vprime )
         vptemp(i) = vupper

         i= NpsiJ
         psilower = 0.5*(xsv(i  ) + xsv(i-1))
         call linr1d(PPSI, xdum, ydum, psilower, vlower, vprime )
         psiupper =      xsv(i  )
         call linr1d(PPSI, xdum, ydum, psiupper, vupper, vprime )
         vptemp(i) = vupper - vlower

      write(7,2202) npsitm,nspc,kcycle,times
c
      write(7,2203) ( anecc(l),l=1,NpsiJ)
      write(7,2203) ( tekev(l),l=1,NpsiJ)
      write(7,2203) ((anicc(l,n),l=1,NpsiJ),n=1,nspc)
      write(7,2203) ((tikev(l,n),l=1,NpsiJ),n=1,nspc)
      write(7,2203) (amass(n),n=1,nspc)
      write(7,2203) (achrg(n),n=1,nspc)
c
      write(7,2203) (voltlp(l),l=1,NpsiJ)
      write(7,2203) (rho(l),l=1,NpsiJ)
      write(7,2203) (vptemp(l),l=1,NpsiJ)
      write(7,2203) (xsv(l),l=1,NpsiJ)

      write(7,2203) (pary(l),l=1,NpsiJ)
      write(7,2203) (ppary(l),l=1,NpsiJ)
      write(7,2203) (gary(l),l=1,NpsiJ)
      write(7,2203) (gpary(l),l=1,NpsiJ)
      write(7,2204) nx,nz,isym,iplim
      write(7,2203) xary(1),xary(nx),zary(1),zary(nz)
c
      write(7,2203) psimin,psilim,xmag,zmag
      write(7,2203) rgzero,bgzero,apl

      write(7,2203) psep(1),xsep(1),zsep(1)
      write(7,2203) psep(2),xsep(2),zsep(2)

      if( isym .eq. 1) then
         write(7,2203) ((PsiGrd(i, j) ,i=1,NumR),
     ^        j=NumZh,NumZ)
      else if (isym .eq. 0) then
         write(7,2203) ((PsiGrd(i, j) ,i=1,NumR),
     ^        j=    1,NumZ)
      else
         stop ' isym must be 1 for up/dn symmetry, or 0'
 2    endif
c
 2201 format(10a8)
 2202 format(3i5,e16.6)
 2203 format(5e16.6)
 2204 format(4i5)
c 
      close(unit=7)
      stop
      END
c                                                                      |
c                                                                      |
c     SBlsc.f(or)  ends                     ---------------------------|
c 
c
c     GetPsiCirc   begins                   ---------------------------|
c                                                                      |
c                                                                      |
      SUBROUTINE GetPsiCirc(r,a,Rmaj,BofA,power,BofR,volume,psi,ifail)
      IMPLICIT NONE
      INTEGER ifail
      REAL    power, pp1, pm0, pm1,pm2,pm3,pm4,pm5,pm6
      REAL    r,a,BofA,Rmaj,psi,volume, roa,u,z, BofR
      ifail = 0
      psi   = 0.0
      BofR  = 0.0
      volume= 0.0
      pp1   = power + 1.
      pm0   = power
      pm1   = power - 1.
      pm2   = power - 2.
      pm3   = power - 3.
      pm4   = power - 4.
      pm5   = power - 5.
      pm6   = power - 6.

      roa = abs(r/a)
      z   = roa*roa
      u   = 1.00

c     volume = 2. * (3.1415926)**2 * Rmaj * abs(r)  ! prior to Feb96
      volume = 2. * (3.1415926)**2 * Rmaj * r * r 

      if      (z .lt. 0.000001 ) then
         BofR = BofA * pp1 * roa
      else if (z .le. 1.) then
         BofR = BofA * ( 1. - (1.- z)**pp1) / roa
      else if (z .gt. 1.) then
         BofR = BofA / roa
      endif

      if      (z .le. 1.) then
         psi = Rmaj * a * BofA * pp1 / 2. * z * 
     ^                               ( 1. -
     ^         pm0*z/((2.   ) * 2.)* ( 1. -
     ^         pm1*z/((3./2.) * 3.)* ( 1. -
     ^         pm2*z/((4./3.) * 4.)* ( 1. -
     ^         pm3*z/((5./4.) * 5.)* ( 1. -
     ^         pm4*z/((6./5.) * 6.))))))
      else 
         psi = 
     ^         Rmaj * a * BofA * alog(roa) +
     ^         Rmaj * a * BofA * pp1 / 2. * u * 
     ^                               ( 1. -
     ^         pm0*u/((2.   ) * 2.)* ( 1. -
     ^         pm1*u/((3./2.) * 3.)* ( 1. -
     ^         pm2*u/((4./3.) * 4.)* ( 1. -
     ^         pm3*u/((5./4.) * 5.)* ( 1. -
     ^         pm4*u/((6./5.) * 6.))))))
      endif
         
      return
      END
      
c                                                                      |
c                                                                      |
c     GetPsiCirc   ends                     ---------------------------|

c
c     ------------------------------------------------------------------
c
      SUBROUTINE linr1d(nx, xdata, ydata, x, y, yp )
      IMPLICIT NONE
      INTEGER j, jguess, nx
      REAL    xdata(nx), ydata(nx), x, y, yp, coef(2)
      REAL    d21, dx
c     LINeaR interpolation in 1D
c
c     The derivative is guaranteed not continuous.  
c     Linear interpolation of interest to avoid fake oscillations
c     in the interpolant.
c
c     here: yint = sum(1 - 2) * coef(j) * ((x - x(1)) ** (j - 1))
c
c                                       find which interval we are in by
c                                       Numerical Recipes HUNT routine
      jguess = -1
      call huntnr(xdata, nx, x, jguess)
      j = jguess
      
      if (j .eq. 0 ) then
        y  = ydata(1)
c       yp = 0.                         yp not of interest since its not
c                                       continuous.
        return
      endif
      if ( j. eq. nx) then
        y  = ydata(nx)
c       yp = 0.                         yp not of interest
        return
      endif
c
c     This is the computation of the slope, and delta-x
            d21     = (ydata(j + 1) - ydata(j    )) /
     ^                (xdata(j + 1) - xdata(j    ))
 
            coef(1) = ydata(j)
            coef(2) = d21
            dx = x - xdata(j)
c
      y  = coef(2) * dx + coef(1)
      yp = coef(2)                    

      return
      END
c
c     ------------------------------------------------------------------
c
      SUBROUTINE huntnr( xx, n, x, jlo )
c     Given an array XX  of length  N , and given a value  X , returns
c     a value  JLO  such that  x  is between  XX(jlo)  and XX(jlo+1).
c     XX  must be monotonic, either increasing or decreasing.
c     JLO=0 or =N is returned to indicate that  X  is out of range.
c     JLO on input is taken as a the initial guess for  JLO on output.
c     Ref:  Numerical Recipes in Fortran, p 91.
      IMPLICIT NONE
      INTEGER inc, jlo, jhi, jm, n
      REAL    xx(n), x
      LOGICAL ascnd
c
c                                       True if table in ascending order.
      ascnd =  xx(n) .gt. xx(1) 
c                                       If input guess is not useful, then
c                                       go immediately to bisection.
      if ( jlo .le. 0 .or. jlo .gt. n ) then
        jlo = 0
        jhi = n + 1
        go to 3
      endif
 
c                                       Set the hunting increment.
      inc = 1
c                                       Hunt up.
      if ( x .ge. xx(jlo)  .eqv.  ascnd ) then
 1      jhi = jlo + inc
c                                       Done hunting; off the table.
        if ( jhi .gt. n ) then
            jhi = n + 1
c                                       Not done hunting.
        else if ( x .ge. xx(jhi)  .eqv.  ascnd ) then
            jlo = jhi
c                                       Double increment, try again.
            inc = inc + inc
            go to 1
c                                       Done hunting, value bracketed.
        endif
c                                       Hunt down.
      else
        jhi = jlo
 2      jlo = jhi - inc
c                                       Done hunting; off the table.
        if ( jlo .lt. 1 ) then
            jlo = 0
c                                       Not done hunting.
        else if ( x .lt. xx(jlo)  .eqv.  ascnd ) then
            jhi = jlo
c                                       So double increment, try again.
            inc = inc + inc
            go to 2
c                                       Done hunting, value bracketed.
        endif
      endif
c                                       Begin bisection phase.
 3    if ( jhi - jlo .eq. 1 ) return
      jm = (jhi + jlo) / 2
      if ( x .gt. xx(jm) .eqv. ascnd ) then
        jlo = jm
      else
        jhi = jm
      endif
      go to 3
      END
c
c     ------------------------------------------------------------------
c

